//	file: Detour.mod
//
// TITLE: Learning to Detour.
// GOAL:  Schema Construction and Dynamics based on SBL.
//
//	REFERENCES: 
//	Refer to Corbacho and Arbib (1995) for details on detour itself.
//	Refer to Corbacho and Arbib (1997) for further details on Learning.
//
//	ARCHITECTURE: 
//	Perceptual Schemas as functional units.
//	Fields corresponding to schema instances;
//	Integration of fields; WTA; Sensorimotor Xform;
//	Motor schemas as functional units.
//
//	SCHEMA DYNAMICS:
//	Relaxation Labeling (Hummel and Zucker, 1983).
//	Control of schemas based on distributed C&C dynamics.
//	Inclusion of NeuroSchemas.
//
//	SIMPLIFICATIONS:
//	Perceptual Schemas (not Neural Implementation).
//	Motor Schemas as functional units.
//
//	SCHEMA CONSTRUCTION:
//	Autonomous Schema Construction in Run time.
// 	Coherence and Performance Maximization.
//
//	AUTHOR: Fernando Jose Corbacho
// DATE (last update): July 3rd, 98        (prev): July 2nd, 98.
//

nslImport nslAllImports;

nslModel Detour ()
{ 
	private World world(200,200,200);
	private Prey prey();
 	private Frog frog(200,200,80,80,80,80,40);

	public void makeConn()
	{
		nslConnect(world.worldXZ,frog.worldXZ);
		nslConnect(world.xb_init,frog.xb_init);
		nslConnect(world.xb_end,frog.xb_end);
		nslConnect(world.yb,frog.yb);
		nslConnect(world.zb,frog.zb);
	
		nslConnect(frog.xf,world.xf);
		nslConnect(frog.yf,world.yf);
		nslConnect(frog.zf,world.zf);
		nslConnect(frog.wangle,world.wangle);

		nslConnect(prey.xw,frog.xw);
		nslConnect(prey.yw,frog.yw);
		nslConnect(prey.zw,frog.zw);

		nslConnect(prey.xw,world.xw);
		nslConnect(prey.yw,world.yw);
		nslConnect(prey.zw,world.zw);
	}
}
