/*
 * nxtVIsionStart.java
 *
 * Created on 30 de noviembre de 2002, 9:11
 */

package VideoGrabber;

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import javax.media.*;
import javax.media.format.*;
import javax.media.protocol.*;
import javax.media.control.*;
import java.util.Enumeration;
import java.util.Properties;
import java.util.Vector;
import javax.media.util.*;
import javax.media.format.RGBFormat;
import javax.media.format.VideoFormat;
import javax.swing.JFrame;
import java.awt.Image;

/**
 *
 * @author  Rodolfo
 */
public class VideoGrabber extends javax.swing.JFrame implements ControllerListener {
    
    private Object waitSync = new Object();
    private boolean stateTransitionOK = true;
    private DataSource              dataSource, clonedDataSource;
    private Processor pr = null;
    private ImageGrabber grabber;
    private Codec[] codec = new Codec[1];
    Format format;
    
    
    /** Creates new form nxtVIsionStart */
    public VideoGrabber() {
        //Manager.setHint(Manager.LIGHTWEIGHT_RENDERER, new Boolean(true));
        initComponents();
        initVideo();
    }
    
    /** Inicializa el video */
    public void initVideo() {
        CaptureDeviceInfo       deviceInfo;
        
        CaptureDeviceDialog captureDeviceDialog = new CaptureDeviceDialog(this, "Configuracin", true);
        captureDeviceDialog.show();
        
        deviceInfo = captureDeviceDialog.getVideoDevice();
        format = captureDeviceDialog.getVideoFormat();
        
        System.out.println( format.toString() );
        
        dataSource = this.createCaptureDataSource( deviceInfo.getName(), format );
        if (!open(dataSource))
            System.exit(0);
        pr.start();
        
        java.awt.Component component = pr.getVisualComponent();
        //component.setBounds(0, 0, 320, 240);
        //panel.add( component );
        this.getContentPane().add( component );
        
        
        this.getContentPane().add( pr.getControlPanelComponent(),  java.awt.BorderLayout.SOUTH );
    }
    
    
    /**Crea el DataSource con el formato especificado*/
    public static DataSource createCaptureDataSource(String strVideoDevice,
    Format videoFormat ) {
        DataSource  dsOne = null;
        DataSource  sources [];
        
        
        if (strVideoDevice == null)
            return null;
        dsOne = initializeCaptureDataSource( null, strVideoDevice, videoFormat );
        if ( dsOne == null )
            return null;
        
        try {
            dsOne.connect();
        }
        catch (java.io.IOException ioe) {
            return null;
        }
        return dsOne;
    }
    
    /**Inicializa la fuente de captura de datos*/
    public static DataSource initializeCaptureDataSource( DataSource ds,
    String deviceName,
    Format format ) {
        MediaLocator        deviceURL;
        CaptureDeviceInfo   cdi;
        DataSource          dataSource = ds;
        FormatControl       formatControls [];
        Format              formats [];
        
        if ( ds == null ) {
            cdi = CaptureDeviceManager.getDevice( deviceName );
            if (cdi == null)
                return null;
            deviceURL = cdi.getLocator();
            System.out.println(deviceURL.toString());
            
            try {
                dataSource = javax.media.Manager.createDataSource(deviceURL);
                dataSource = javax.media.Manager.createCloneableDataSource( dataSource );
                if (dataSource == null)
                    return null;
            } catch (NoDataSourceException ndse) {
                System.out.println("No se pudo crear dataSource");
                return null;
            } catch (java.io.IOException ioe) {
                System.out.println("No se pudo crear dataSource");
                return null;
            }
        }
        
        if ( format == null )
            return dataSource;
        
        if ( format != null  &&  !(dataSource instanceof CaptureDevice) )
            return null;
        
        formatControls = ((CaptureDevice) dataSource).getFormatControls();
        if ( formatControls == null  ||  formatControls.length == 0 )
            return null;
        for ( int i = 0; i < formatControls.length; i++ ) {
            if ( formatControls[i] == null )
                continue;
            formats = formatControls[i].getSupportedFormats();
            if (formats == null)
                return null;
            if (matches(format, formats) != null) {
                formatControls[i].setFormat(format);
                return dataSource;
            }
        }
        return null;
    }
    
    
    
    private static Format matches( Format format, Format supported[] ) {
        if ( supported == null )
            return null;
        for ( int i = 0;  i < supported.length;  i++ ) {
            if ( supported[i].matches(format) )
                return supported[i];
        }
        return null;
    }
    
    public boolean open(DataSource ds) {
        
        try {
            pr = Manager.createProcessor(ds);
        } catch (Exception e) {
            System.out.println("No se pude crear un Processor: " + e);
            return false;
        }
        
        pr.addControllerListener(this);
        
        // Poner al procesador en el estado configure
        pr.configure();
        if (!waitForState(pr.Configured)) {
            System.out.println("Fallo en poner el Processor en Configure.");
            return false;
        }
        
        // Para que se pueda usar como Player.
        pr.setContentDescriptor(null);
        
        // Obtener los track controls.
        TrackControl tc[] = pr.getTrackControls();
        
        if (tc == null) {
            System.out.println("Fallo en obtener los Track Control del Processor.");
            return false;
        }
        
        // Buscar el TrackControl para el video
        TrackControl videoTrack = null;
        
        for (int i = 0; i < tc.length; i++) {
            if (tc[i].getFormat() instanceof VideoFormat) {
                videoTrack = tc[i];
                break;
            }
        }
        
        if (videoTrack == null) {
            System.out.println("El medio de entrada no tiene pista de Video.");
            return false;
        }
        
        // Instanciar y poner los Codec en el flujo del stream.
        try {
            //Iniciliazar el codec para obtener las imagenes
            grabber = new ImageGrabber((VideoFormat)format);
            codec[0] = grabber;
         
            videoTrack.setCodecChain(codec);
        } catch (UnsupportedPlugInException e) {
            System.out.println("El procesador no soporta Codecs.");
        }
        
        // Poner al Processor en Realize
        pr.prefetch();
        if (!waitForState(pr.Prefetched)) {
            System.out.println("Se fallo en poner el Procesor en Realize.");
            return false;
        }
        return true;
    }
    
    boolean waitForState(int state) {
        synchronized (waitSync) {
            try {
                while (pr.getState() != state && stateTransitionOK)
                    waitSync.wait();
            } catch (Exception e) {}
        }
        return stateTransitionOK;
    }
    
    
    
    void stop() {
        if (pr!=null) {
            pr.stop();
            pr.deallocate();
        }
        if (dataSource != null){
            dataSource.disconnect();
        }
        
    }
    
    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    private void initComponents() {//GEN-BEGIN:initComponents

        addWindowListener(new java.awt.event.WindowAdapter() {
            public void windowClosing(java.awt.event.WindowEvent evt) {
                exitForm(evt);
            }
        });

        pack();
        java.awt.Dimension screenSize = java.awt.Toolkit.getDefaultToolkit().getScreenSize();
        setSize(new java.awt.Dimension(367, 244));
        setLocation((screenSize.width-367)/2,(screenSize.height-244)/2);
    }//GEN-END:initComponents
    
    /** Exit the Application */
    private void exitForm(java.awt.event.WindowEvent evt) {//GEN-FIRST:event_exitForm
        stop();
        System.exit(0);
    }//GEN-LAST:event_exitForm
    
    /**
     * @param args the command line arguments
     */
    public static void main(String args[]) {
        new VideoGrabber().show();
    }
    
    public void controllerUpdate(javax.media.ControllerEvent evt) {
        if (evt instanceof ConfigureCompleteEvent ||
        evt instanceof RealizeCompleteEvent ||
        evt instanceof PrefetchCompleteEvent) {
            synchronized (waitSync) {
                stateTransitionOK = true;
                waitSync.notifyAll();
            }
        } else if (evt instanceof ResourceUnavailableEvent) {
            synchronized (waitSync) {
                stateTransitionOK = false;
                waitSync.notifyAll();
            }
        } else if (evt instanceof EndOfMediaEvent) {
            pr.close();
            System.exit(0);
        }
    }
    
    
    public java.awt.Image getImage() {
        return this.grabber.getImage();
    }
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    // End of variables declaration//GEN-END:variables
    
}
