/*
 * AngleCurve.java
 *
 * Created on 27 de abril de 2004, 17:37
 */

package cannes.NSLWorld;

import java.lang.Math;
import nslj.src.lang.NslDouble2;
/**
 *
 * @author  sylvain
 */
public class AngleCurve {
    private int angleMin, angleMax, deltaAngle;
    private int values[];
    private int valuesNumber; // size of the array
    
    /** Creates a new instance of AngleCurve */
    public AngleCurve(int angleMin, int angleMax, int deltaAngle) {
        int i;
        
        this.angleMin = angleMin;
        this.angleMax = angleMax;
        this.deltaAngle = deltaAngle;
        valuesNumber = (int)((angleMax - angleMin) / deltaAngle);
        values = new int[valuesNumber];
        
        for(i = 0; i < valuesNumber ; i++)
            values[i] = 0;
    }
    
    /*
    public AngleCurve(int[] valeurs, int[] angles, int deltaAngle) {
        int nbValeurs;
        int i, j;
        
        nbValeurs = valeurs.length;
        if(angles.length != nbValeurs)
            throw new java.security.InvalidParameterException("Les deux tableaux nont pas la mme taille");
        
        angleMin = angles[0];
        angleMax = angles[nbValeurs-1];
        this.deltaAngle = deltaAngle;
        valuesNumber = (int)((angleMax - angleMin) / deltaAngle);
        values = new int[valuesNumber];
        
        j = 0 ;
        for(i = 0 ; i < valuesNumber ; i++)
        {
            while(angleMin + deltaAngle * i > angles[j])
                j++;
            if(angleMin + deltaAngle * i == angles[j])
                values[i] = valeurs[j];
            else
                values[i] = valeurs[j-1]
                    + (angleMin + deltaAngle * i - angles[j-1]) 
                    * ((valeurs[j] - valeurs[j-1])/(angles[j] - angles[j-1]));
        }
    }*/
    
    public void load(int[] valeurs, int[] angles) {
        int nbValeurs;
        int i, j;
        
        nbValeurs = valeurs.length;
        if(angles.length != nbValeurs)
            throw new java.security.InvalidParameterException("Les deux tableaux nont pas la mme taille");

        for(i = 0 ; i < valuesNumber ; i++)
            values[i] = 0;

        i = (int)((angles[0] - angleMin) / deltaAngle);
        j = 0;
        
        while(j < nbValeurs && i < valuesNumber)
        {
            while(j < nbValeurs && angleMin + deltaAngle * i > angles[j])
                j++;
            if(j < nbValeurs)
            {
                if((angleMin + deltaAngle * i == angles[j]) || (j == 0))
                    values[i] = valeurs[j];
                else
                    values[i] = valeurs[j-1]
                        + (angleMin + deltaAngle * i - angles[j-1]) 
                        * ((valeurs[j] - valeurs[j-1])/(angles[j] - angles[j-1]));
            }
            i++;
        }
    }
    
    public int[][] getMatrix() {
        int[][] retour = new int[getValuesNumber()][2];
        
        for(int i = 0 ; i < getValuesNumber() ; i++)
        {
            retour[i][0] = values[i];
            retour[i][1] = getAngleMin() + i * getDeltaAngle();
        }
        
        return retour;
    }
    
    public int getAngleMin() {
        return angleMin;
    }

    public int getAngleMax() {
        return angleMax;
    }
    
    public int getDeltaAngle() {
        return deltaAngle;
    }
    
    public int getValuesNumber() {
        return valuesNumber;
    }
    
    public int getValue(int angle) {
        if(angle < angleMin || angle > angleMax || ((angle-angleMin) % deltaAngle) != 0)
            throw new java.security.InvalidParameterException("angle incorrect");
        return values[(int)((angle - angleMin) / deltaAngle)];
    }
    
    public NslDouble2 getNSLDouble2() {
        double d[][] = new double[getValuesNumber()][2];
        
        for(int i = 0 ; i < getValuesNumber() ; i++)
        {
            d[i][0] = values[i];
            d[i][1] = angleMin + i * deltaAngle;
        }
        
        return new NslDouble2("Atraction", d);
    }
}
